/***************************************************************************
                          hobit.h  -  description
                             -------------------
    begin                : Do Nov 4 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef HOBIT_H
#define HOBIT_H


// std includes
#include <string>
#include <list>
using namespace std;

// qt includes
#include <qwidget.h>
#include <qmessagebox.h>
#include <qtextedit.h>
#include <qstring.h>
#include <qdatetime.h>
#include <qptrvector.h>

// hobit includes
#include "hobit_interactor.h"
#include "hobit_auth.h"
#include "hobit_validator.h"
#include "hobit_connector.h"
//#include "hobit_transfertemplate.h"
#include "databaseaccess.h"
#include "transaction.h"


#include "accountjobfactory.h"
#include "adminjobfactory.h"

// openhbci2 includes
#include <openhbci2.h>
using namespace HBCI;

// AqMoney includes
#include "aqmoney2/aqmapi.h"
#include "aqmoney2/accountjobs.h"
#include "aqmoney2/adminjobs.h"

//gwenhywfar includes
#include <gwenhywfar/directory.h>





/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif

class AccountJobFactory;
class AdminJobFactory;
class DlgLogMessages;
class HoBIT_Auth;
class HoBIT_TransferTemplate;

/** The HoBIT-Class is the Main Interface between the Money-Penny Client
  * and AqMoney. HoBIT contains methods for all hbci-usecases, that
  * are executed by moneypenny. 
  *
  * <b> How to use this class: </b>
  *
  * First you have to create an instance of the HoBIT-class:
  *
  * <p> HoBIT* hobit = new HoBIT(...);</p>
  *
  * Look at the description of the base constructor to find out which 
  * parameters will be needet. <br>
  *
  * After you have an existing instance of the HoBIT-Class, you have to
  * create a new user, calling the hobit->createUser(...); <br>
  *
  * Now you can start to create jobs calling creator methods:
  *
  * <p>
  * hobit->createJobGetKeys(); <br>
  * hobit->createJobGetSystemId(); <br>
  * hobit->createJobGetBalance(); <br>
  * hobit->createJobGetTransactions(...); <br>
  * hobit->createJobStartStandingOrder(...); <br>
  * ... and so on...
  * </p>
  *
  * The HoBIT stores jobs internally in 2 different job queues (AdminJobs and AccountJobs).
  * To execute stored hbci-jobs you will just have to perform: 
  *
  * <p> hobit->executeJobQueue();
  *
  * This will execute both queues (AdministrationJobs and AccountJobs)
  * <p>
  * <i> <b>
  * To work with this class and the values Hobit Methods return 
  * you will also have to understand the following classes: <br> <br>
  *
  * From OPENHBCI2:	HBCI::Pointer, HBCI::Date, HBCI::Value, HBCI::User <br>
  * From HoBIT:		Transaction, StandingOrder, DatedTransfer <br> <br>
  *
  * And of course: Base knowledge about the qt library, because this is 
  * an OpenHBCI-QT-Interface class. 
  *
  * 
  * </b> </i>
  *<br>
  *@author Marcel Mulch
  *
  *@date 22.10.2004
  */
class HoBIT : public QObject {
 Q_OBJECT
signals:
 void logTextToShow(QString logText); 
public: 

    /**
    *  base constructor for the HoBit QT/OpenHBCI-Interface
    *
    *  In Order to work with HoBIT properly, you will have to
    *  create an HoBIT-Objects, which creates jobs and executes the
    *  job queues. HoBIT is the most important interface between
    *  the graphical user client and AqMoney2
    *  
    * @param readonly set this parameter true, to forbid hobit to perform jobs
    *		      which have writing access on our bank account. Those jobs
    *		      are (in normal homebanking environment) every kind of 
    *		      transactions, the changing of keys and systemid...etc
    *
    * @param parent This is the qwidget which uses HoBIT. in most cases this
    *		    will be the main widget of the hbci-client
    *
    * @param openhbcifile name of the file, which shall store the hbci-configuration
    *			  please consider, that this is ONLY THE FILENAME without any
    *			  path indication. The file will be created in the users main
    *			  directory. 
    *
    * @param aqmoneyfile    name of the file, which shall store the accountinformation
    *			  please consider, that this is ONLY THE FILENAME without any
    *			  path indication. The file will be created in the users main
    *			  directory. 
    *
    * @param datadir the name of the directory where logfiles and transaction files will
    *		     be stored. Please consider, that this should be only the name of the
    *		     directory (for example ".moneypenny"). The directory will be created 
    *		     in the users home directory. 
    *
    * @param subdir the name of the folder in home directory where the moneypenny files are
    *
    * @param applicationName The name of your application (for example "MoneyPenny"). HoBIT
    *			     and AqMoney need this parameter to identify the client at the 
    *			     bank. 
    *
    * @param crypt     HoBIT_Crypt implements cryptographic functions used by HoBIT to encrypt 
    *		       and decrypt user data and files. The OpenSSL implementation of the blowfish 
    *		       algorithm is used. HoBIT needs a crypt object so it can encrypt files
    *		       with an user chosen password
    *		    	
    * @author Marcel Mulch
    *
    * @date 22.11.2004
    */
    HoBIT(bool readonly,
          QWidget *parent,
          const string &openhbcifile,
          const string &aqmoneyfile,
	  const string &templatefile,
          const string &hbcidatadir,
          const string &configdatadir,
          const string &applicationname,
          HoBIT_Connector * connector);
	
  /** base destructor (not of importance) 
  */	    
  virtual ~HoBIT();

  /** This method executes the HoBIT-JobQueues  <br>
  *   HoBIT stores Jobs in 2 different JobQueues: AdminJobs and AccountJobs
  *   Every created Job is put into one of these JobQueues
  *   Jobs are not executed before this method is called. 
  */
  HobitError executeJobQueue();
  

  /** base method for creating a new customer 
  *   creating a customer is important to get a working hbci-environment.
  *   After creating a customer you should create and execute the jobs
  *   getKeys, getSystemId and getAccounts to get a full working hbci 
  *   environment.
  * 
  * @param mediumname path of the security medium. this can be the path where
  *		      the chipcard is mounted or the path where the rsa-file is 
  *		      stored
  *
  * @param mediumtype  the type of the medium. OpenHBCI (and hbci in general)
  *                    differs between 2 types of security mediums. The
  *                    first one is RDH-File. The other one is DEC-Chipcard
  *                    With this parameter you can tell HoBIT (and OpenHBCI)
  *                    which type of medium you are using. <br> <br>
  *
  *                    If you are using RDH-File:   "RDHFile" <br>
  *                    If you are using DEC-Card:   "DECCard"  <br>
  *
  * @param readonly    You can decide if the security medium should be set to
  *                    readonly. If you want so set readonly to true
  *
  * @param country     This is the countrycode. It is needed to tell HBCI the
  *                    location of your bank. This parameter should be 280 if
  *                    you are trying to connect to a german bank.
  *
  * @param instid      The institute code. Every bank has a special code, what
  *                    is unique and identifies it. In Germany this bank code
  *                    is just called Bankleitzahl.
  *
  * @param userid      In HBCI users have an Identification code too. Because
  *                    Users get this code from their bank I do not have to
  *                    explain it. Look into the HBCI-Specification if unsure.
  *
  * @param customerid  In HBCI customers have an Identification code too. Because
  *                    Customers get this code from their bank I do not have to
  *                    explain it. Look into the HBCI-Specification if unsure
  *                    In most cases the customerid will be equal to the user id.
  *
  *
  * @param server      this might be the url or the ip-adress of the bank server
  */
  HobitError createUser(const string &mediumname,
                                  const string &mediumType,
                                  bool readonly,
                                  int country,
                                  const string &instid,
                                  int hbciVersion,
                                  const string &userid,
                                  const string &customerid,
                                  const string &server);
   
   
  /** Reads the data from the chipcard. There MAY be userid, bankid, server. Returns the number of valid user data sets.
  */
  int readCardData(int & country, QString & bankId, QString & userId, QString & server, QString & cardnumber);
  
   /** base method for creating a new customer using a previously
    *   configured DDV-Card
    */
 /* HobitError importUserDDV(bool readonly,
                           int country,
                           const string &instid,
                           const string &userid,
                           const string &customerid,
                           const string &server);
   */                               

  /** 
  *   Loads the OpenHBCI environment when the Program is started 
  *   This method reads the configuration files stored in the 
  *   users home directory
  */
  bool loadEnvironment();

  
  /** Saves the OpenHBCI environment when the Program is shut 
  *   This method writes the environment to the configurationfiles 
  *   stored in the users home directory
  */
  bool saveEnvironment();

  
  /** adds a logtext */
  void showLogText(QString string);


  
  //-------------------------------------------------------------------------
  //  DB-AccessOperations
  //-------------------------------------------------------------------------

  /** Returns a std::list which includes all registered users as a struct
  * 
  *   Have a look to (OpenHBCI)::User to work with it
  */
  list<Pointer<User> > listUsers();

  /** removes all existing banks and users from the gwen database (config-files)
  */
  void deleteAllBanks();
  
  
  /** searches the api for a special bank */
  Pointer<HBCI::Bank> findBank(int country, const string& instId);


  /** changes the HBCI version to be used*/
  bool changeHbciVersion(int value);
  
  
  /** creates a new account with the overgiven values
  * this procedure will for example be done, when 
  * the retrieving of the account list from the 
  * bank server fails 
  *
  * @param name: Name of the account owner
  * @param accountId: the account number
  * @param accountSubId: Hbci allows subids for accounts 
  * 			 this parameter is normaly not needed
  * @param accountName:  you can give your account an description
  *
  */			   
  virtual HobitError addAccount(const string& name = "", 
			        const string &accountId = "", 
			        const string &accountSubId = "", 
			        const string& accountName = "");
  
  /** Deletes account accountId by deleting its directory. (There is no deleteAccount function in openhbci) 
  */
  void deleteAccount(QString accountId);
  
  
  /** changes the account description */
  void changeAccountName(const string& accid, const string& name);



    /** get a list of all accounts 
    *   Have a look to AqMoney::Account, to get information about how to read
    *   out the accounts data
    */
  list<Pointer<Account> > listAccounts();
  
  
  
  /** searches the aqmapi for a single account
    */
  Pointer<Account> findAccount(const string &accid);



    /** returns a list of transactios according to a single account 
     *  during a specified time period. To get all transactions stored,
     *  call listTransactions(0, "*", "*", "*", Date(), Date());
     *
     * @param country the banks countrycode (280 for a german bank)
     *		      set to 0 for all banks
     *
     * @param bankCode the bankId (in german Bankleitzahl) of a bank
     *	  	       set to "*" for all stored banks
     *
     * @param bankCode the accountId of an account
     *	  	       set to "*" for all stored accounts of the bank
     *
     * @param accounSubId in HBCI accounts can have subids. in most 
     * 			  cases you should overgive "" here
     *
     * @param fromdate the time periods lower frontier. Have a look to
     *		       OpenHBCI::Date to find out how a Date Object works
     *		       Overgive an empty date: Date() if you do not want
     *		       to set a lower border 
     *		       
     * @param todate   the time periods upper frontier. Have a look to
     *		       OpenHBCI::Date to find out how a Date Object works
     *		       Overgive an empty date: Date() if you do not want
     *		       to set an upper border
     */
  list<Pointer <Transaction> > listTransactions(int country,
                                                const string &bankCode,
                                                const string &accountId,
                                                const string &accountSubId,
                                                const QDate &fromDate,
                                                const QDate &toDate);

 

  /** returns a list of dated transfers 
  *   concerning a single account. Call listDatedTransfers(0, "*", "*", "*") to
  *   get all dated transfers from all accounts.
  *
  * @param country the banks countrycode (280 for a german bank)
  *		      set to 0 for all banks
  *
  * @param bankCode the bankId (in german Bankleitzahl) of a bank
  *	  	       set to "*" for all stored banks
  *
  * @param bankCode the accountId of an account
  *	  	       set to "*" for all stored accounts of the bank
  *
  * @param accounSubId in HBCI accounts can have subids. in most 
  * 			  cases you should overgive "" here
  */                                                                   
  list <Pointer <DatedTransfer> >  listDatedTransfers(int country,
                                                      const string &bankCode,
                                                      const string &accountId,
                                                      const string &accountSubId);



  /** returns a list of standing orders
  *   concerning a single account. Call listStandingOrders(0, "*", "*", "*") to
  *   get all standing orders from all accounts.
  *
  * @param country the banks countrycode (280 for a german bank)
  *		      set to 0 for all banks
  *
  * @param bankCode the bankId (in german Bankleitzahl) of a bank
  *	  	       set to "*" for all stored banks
  *
  * @param bankCode the accountId of an account
  *	  	       set to "*" for all stored accounts of the bank
  *
  * @param accounSubId in HBCI accounts can have subids. in most 
  * 			  cases you should overgive "" here
  */                                                                   
  list <Pointer <StandingOrder> >  listStandingOrders(int country,
                                                      const string &bankCode,
                                                      const string &accountId,
                                                      const string &accountSubId);
                                                                             



  /** returns a std::string, which describes the iniletter 
  */                                                                   
  string generateIniLetter();

  
  /** checks and returns the date of the last transaction stored 
  *   -> QDate
  * !!!!WARNING: This date may be in the future!!!!
  */
  QDate getLastDateOfTransactions();
  
  
  
  /**
  * returns a list of all known banks. Banks are 
  * returned as HoBIT_Bank-objects
  * Please note that ktoblzcheck must be installed
  * to use this function,
  */
  list<HoBIT_Bank> listOfAllKnownBanks();
  
  /**
  * returns a list of all known banks that support HBCI. Banks are 
  * returned as HoBIT_Bank-objects
  * Please note that fints_isnstute.csv must be installed
  * to use this function,
  */
  list<HoBIT_Bank> listOfKnownHbciBanks();
  
  
   /** returns the hash fingerprint of the institute after resceiving the bank servers public keys 
 */
  string getBankFingerPrint(int country, const string& bankcode, const string& customerId);

  /** Returns the name (url) of the bank server
  */
  QString getBankServer(int country, const string &BankId);
 
    /** returns the fingerprint of the current keypair
    */
  string getMyOwnFingerPrint(int country, const string& bankcode, const string& customerId); 
  
  /** adds a transfer template to database
  */
  void addTransferTemplate(HoBIT_TransferTemplate * transferTemplate);
 			      
  /** deletes a transfer template 
  */
  void deleteTransferTemplate(const string& ownAccountId,
  			      const string& otherName, 
			      const string& otherAccountId,
			      const string& otherBank,
			      double value,
			      const QStringList& purpose);
    

  /** returns all transfer templates 
  */
  list<Pointer<HoBIT_TransferTemplate> > getTransferTemplates();   			      
			      
			      
  //-------------------------------------------------------------------------
  //  Administration Jobs
  //------------------------------------------------------------------------
  
  /** returns the account job queue 
  */
  list<Pointer<AdminJob> > getAdminJobQueue();
  
  /** deletes a certain admin job from the adminjob queue 
  */
  void deleteAdminJob(int number);
  
  /** 
   *   this method creates a job to get the servers public keys and puts 
   *   it into the Admin Job Queue. 
   */
  HobitError createJobGetKeys();

  /** creates a new job, which will retrieve a new system id 
   *  The system id is needed to identify the client system 
   *  to the bank server.
   * This job is stored in the Admin Job Queue
   */ 
  HobitError createJobGetSystemId();
 
  /** creates a job, which retrieves a list of all hbci accounts a 
    * bank customer has
    * This job is stored in the Admin Job Queue
    */
  HobitError createJobGetAccounts();

  /** creates a job send keys which will deliver the own public 
    * keys to the bank server 
    * This job is stored in the Account Job Queue
    */
  HobitError createJobSendKeys();

  /** creates a job, which retrieves status information from our bank
    * This job is of less importance, but has been added 
    * fore completeness
    */
  HobitError createJobGetReports();
    
  /** creates a new security medium using the given information 
    * @ param mtype the medium type
    * @ param readonly decides whether the medium will be mounted readonly or not
    * @ param country the countrycode of the bank
    * @ param bankId the bankcode (bankleitzahl) of the institute
    * @ param userId the userId
    * @ param name the path of the new security file
    */
  HobitError createNewMedium    ( const string & mtype, 
				  bool readonly, 
				  int country, 
				  const string & bankId, 
				  const string & userId, 
				  const string & keyname,
                                  const QString & newPassword);
                                  

  //-------------------------------------------------------------------------
  //  Account Jobs
  //------------------------------------------------------------------------

  
  /** returns the account job queue */
  list<Pointer<AccountJob> > getAccountJobQueue();
  
  /** deletes a certain account job from the accountjob queue */
  void deleteAccountJob(int number);

   /** creates a job get balance which will be executed directly */
  //HobitError createAndSendJobGetBalance();

  /** creates a job get balance to retrieve the balances of all 
      accounts hobit maintains. The job will be stored in hobits 
      Account job queue for later sending.  */
  HobitError createJobGetBalance();


  /** creates a job get get transaction which creates a job for retrieving all transactions
    *  belonging to a specific account during a defined time period 
    *  this job is send directly 
    *
    * @ param fdate the date of the first transaction to get
    * @ param tdate the date of the last transaction to get
    * @ param accountId the id of the account to get transactions for
    */
  //HobitError createAndSendJobGetTransactions(Date fdate, Date tdate, const string& accountId);

  /** creates a job get get transaction which creates a job for retrieving all transactions
    *  belonging to a specific account during a defined time period 
    *  this job will be stored for later sending
    *
    * @ param fdate the date of the first transaction to get
    * @ param tdate the date of the last transaction to get
    * @ param accountId the id of the account to get transactions for
    */
  HobitError createJobGetTransactions(Date fdate, Date tdate, const string& accountId);

  
   /** creates a job get get dated transfers which creates a job for retrieving 
    *  all dated transfers belonging to a specific account during a defined time period 
    *  this job is send directly 
    *  you can also call createAndSendJobGetDatedTransfer() without parameters, if no
    *  special time period or account number is wanted
    *
    * @ param fdate the date of the first transaction to get
    * @ param tdate the date of the last transaction to get
    * @ param accountId the id of the account to get transactions for
    */
  /*HobitError createAndSendJobGetDatedTransfers(Date fdate = Date(),
                                               Date tdate = Date(),
                                               const string& accountId = "*");
 */
  
  /** creates a job get get dated transfers which creates a job for retrieving 
    *  all dated transfers belonging to a specific account during a defined time period 
    *  this job is stored in the account job queue for later sending
    *  you can also call createJobGetDatedTransfer() without parameters, if no
    *  special time period or account number is wanted
    *
    * @ param fdate the date of the first transaction to get
    * @ param tdate the date of the last transaction to get
    * @ param accountId the id of the account to get transactions for
    */
  HobitError createJobGetDatedTransfers(Date fdate = Date(),
                                        Date tdate = Date(),
                                        const string& accountId = "*");

    
    /** creates a job get get standing orders which creates a job for retrieving 
    *  all standing orders belonging to a specific account 
    *  this job will be sended directly
    *  you can also call createAndSendJobGetStandingOrders() without parameters
    *  to get all dated transfers for all accounts
    *
    * @ param accountId the id of the account to get transactions for
    */
   //HobitError createAndSendJobGetStandingOrders(const string& accountId = "*");
                                                         
    /** creates a job get get standing orders which creates a job for retrieving 
    *  all standing orders belonging to a specific account 
    *  this job will be sended directly
    *  you can also call createAndSendJobGetStandingOrders() without parameters
    *  to get all dated transfers for all accounts
    *
    * @ param accountId the id of the account to get transactions for
    */
  HobitError createJobGetStandingOrders(const string& accountId = "*");



  /** creates a new job for starting a single transfer
   *  this job will be send to the bank server directly
   *
   * @param skipbad set this parameter true if you want hobit to skip
   *       	    transactions which are supposed to be bad
   *		    this happens for example if an unknown bank id is 
   * 	 	    overgiven. Please keep in mind that a testbankserver (gad?)
   *	 	    will be supposed to be false because ktoblzcheck does not
   *		    know its bank id
   *
   * @param country the countrycode of our own bank (280 for germany)
   * 
   * @param bankCode the bankid of our bank
   *
   * @param accountId the account from where the transaction comes
   * 
   * @param accountSubId if the account has a subid (say "*" or "" if none)
   *
   * @param otherCountry the countrycode of the account which will resceive the transaction
   *			 (280 if a german bank)
   *
   * @param otherBankCode the bank code of the resceivers bank
   *
   * @param otherAccount  the id of the account which is supposed to resceive the transaction 
   *
   * @param otherName the name of the person or company that owns the resceiver account
   *		      in hbci standard [Name|NameExtension]
   *
   * @param purpose the purpose of the transaction (can consist of several purpose lines
   *
   * @param value the value to be transfered (look openhbci::value for more information)
   *
   * @param textkey textkey to be used for the transaction. A valid textkey can be
   *		    found in the bank parameter data
   * 
   * @param date the date for the transaction to be performed. Overgive an empty "Date()" to
   *		 create a transaction that will be executed directly. Otherwise, if you want 
   *		 the transaction to be dated overgive a valid Date(day, month, year)
   *
   * @author marcel mulch
   *
   * @date 25.11.2004
   */	      
  /*HobitError createAndSendJobSingleTransfer(bool skipBad,
                                            int country,
                                            const string &bankCode,
                                            const string &accountId,
                                            const string &accountSubId,
                                            int otherCountry,
                                            const string& otherBankCode,
                                            const string& otherAccount,
                                            const list<string> &otherName,
                                            const list<string> &purpose,
                                            const  Value&  value,
                                            int textkey,
                                            Date date);
*/
 
 /** creates a new job for starting a single transfer
   *  this job will be stored for later sending
   *
   * @param skipbad set this parameter true if you want hobit to skip
   *       	    transactions which are supposed to be bad
   *		    this happens for example if an unknown bank id is 
   * 	 	    overgiven. Please keep in mind that a testbankserver (gad?)
   *	 	    will be supposed to be false because ktoblzcheck does not
   *		    know its bank id
   *
   * @param country the countrycode of our own bank (280 for germany)
   * 
   * @param bankCode the bankid of our bank
   *
   * @param accountId the account from where the transaction comes
   * 
   * @param accountSubId if the account has a subid (say "*" or "" if none)
   *
   * @param otherCountry the countrycode of the account which will resceive the transaction
   *			 (280 if a german bank)
   *
   * @param otherBankCode the bank code of the resceivers bank
   *
   * @param otherAccount  the id of the account which is supposed to resceive the transaction 
   *
   * @param otherName the name of the person or company that owns the resceiver account
   *		      in hbci standard [Name|NameExtension]
   *
   * @param purpose the purpose of the transaction (can consist of several purpose lines
   *
   * @param value the value to be transfered (look openhbci::value for more information)
   *
   * @param textkey textkey to be used for the transaction. A valid textkey can be
   *		    found in the bank parameter data
   * 
   * @param date the date for the transaction to be performed. Overgive an empty "Date()" to
   *		 create a transaction that will be executed directly. Otherwise, if you want 
   *		 the transaction to be dated overgive a valid Date(day, month, year)
   *
   * @author marcel mulch
   *
   * @date 25.11.2004
   */	      
  HobitError createJobSingleTransfer(bool skipBad,
                                                       int country,
                                                       const string &bankCode,
                                                       const string &accountId,
                                                       const string &accountSubId,
                                                       int otherCountry,
                                                       const string& otherBankCode,
                                                       const string& otherAccount,
                                                       const list<string> &otherName,
                                                       const list<string> &purpose,
                                                       const  Value&  value,
                                                       int textkey,
                                                       Date date);

   
   
   /** Creates a job to delete a dated transfer. This job will be stored for later delivery.
    * You will have to create a DatedTransfer-Object first or overgive a transaction you 
    * have gotten from the listDatedTransfers() method. Look into the dated transfer 
    * class for more information
    *
    * @param dt dated transfer to be deleted
    */
  HobitError createJobDeleteDatedTransfer(Pointer<DatedTransfer> dt);

  
  
  
  
  /** creates a new job for starting a standing order
   *  this job will be delivered directly.
   *
   * @param skipbad set this parameter true if you want hobit to skip
   *       	    transactions which are supposed to be bad
   *		    this happens for example if an unknown bank id is 
   * 	 	    given. Please keep in mind that a testbankserver (gad)
   *	 	    will be supposed to be false because ktoblzcheck does not
   *		    know its bank id
   *
   * @param country the countrycode of our own bank (280 for germany)
   * 
   * @param bankCode the bankid of our bank
   *
   * @param accountId the account from where the transaction comes
   * 
   * @param accountSubId if the account has a subid (say "*" or "" if none)
   *
   * @param otherCountry the countrycode of the account which will resceive the transaction
   *			 (280 if a german bank)
   *
   * @param otherBankCode the bank code of the resceivers bank
   *
   * @param otherAccount  the id of the account which is supposed to resceive the transaction 
   *
   * @param otherName the name of the person or company that owns the resceiver account
   *		      in hbci standard [Name|NameExtension]
   *
   * @param purpose the purpose of the transaction (can consist of several purpose lines
   *
   * @param value the value to be transfered (look openhbci::value for more information)
   *
   * @param textkey textkey to be used for the transaction. A valid textkey can be
   *		    found in the bank parameter data
   * 
   * @param date  setting this parameter will not have any deeper effect. Howewer: Because
   *		  of AqMoney's internal structure a date can be set here, but is of no real
   *		  importance. More important are the fromdate, the todate the turnus and the day
   *
   * @param firstdate the date of first execution of the transaction
   *
   * @param lastdate the date of the last execution. If you do not want to set a border 
   *	             overgive a nulldate Date()
   *
   * @param turnus the interval of repetition. Defines together with the weekly parameter
   *		   how often the transaction will be done. 
   *		   For example Turnus = 3 - Weekly = false will perform the transaction every
   *		   third month
   *
   * @param weekly defines the unit of the turnus.
   *        if true the used unit is weeks. If false it will be month instead
   *
   * @param day defines the day of the month, when the transaction will be executed
   *
   * @author marcel mulch
   *
   * @date 25.11.2004
   */	
  /*HobitError createAndSendJobStartStandingOrder(bool skipBad,
                                                                           int country,
                                                                           const string &bankCode,
                                                                           const string &accountId,
                                                                           const string &accountSubId,
                                                                           int otherCountry,
                                                                           const string& otherBankCode,
                                                                           const string& otherAccount,
                                                                           const list<string> &otherName,
                                                                           const list<string> &purpose,
                                                                           const  Value&  value,
                                                                           int textkey,
                                                                           const Date& date,
                                                                           const Date& firstdate,
                                                                           const Date& lastdate,
                                                                           const int turnus,
                                                                           bool weekly,
                                                                           int day);

  */
 /** creates a new job for starting a standing order
   *  this job will be stored for later sending
   *
   * @param skipbad set this parameter true if you want hobit to skip
   *       	    transactions which are supposed to be bad
   *		    this happens for example if an unknown bank id is 
   * 	 	    given. Please keep in mind that a testbankserver (gad)
   *	 	    will be supposed to be false because ktoblzcheck does not
   *		    know its bank id
   *
   * @param country the countrycode of our own bank (280 for germany)
   * 
   * @param bankCode the bankid of our bank
   *
   * @param accountId the account from where the transaction comes
   * 
   * @param accountSubId if the account has a subid (say "*" or "" if none)
   *
   * @param otherCountry the countrycode of the account which will resceive the transaction
   *			 (280 if a german bank)
   *
   * @param otherBankCode the bank code of the resceivers bank
   *
   * @param otherAccount  the id of the account which is supposed to resceive the transaction 
   *
   * @param otherName the name of the person or company that owns the resceiver account
   *		      in hbci standard [Name|NameExtension]
   *
   * @param purpose the purpose of the transaction (can consist of several purpose lines
   *
   * @param value the value to be transfered (look openhbci::value for more information)
   *
   * @param textkey textkey to be used for the transaction. A valid textkey can be
   *		    found in the bank parameter data
   * 
   * @param date  setting this parameter will not have any deeper effect. Howewer: Because
   *		  of AqMoney's internal structure a date can be set here, but is of no real
   *		  importance. More important are the fromdate, the todate the turnus and the day
   *
   * @param firstdate the date of first execution of the transaction
   *
   * @param lastdate the date of the last execution. If you do not want to set a border 
   *	             overgive a nulldate Date()
   *
   * @param turnus the interval of repetition. Defines together with the weekly parameter
   *		   how often the transaction will be done. 
   *		   For example Turnus = 3 - Weekly = false will perform the transaction every
   *		   third month
   *
   * @param weekly defines the unit of the turnus.
   *        if true the used unit is weeks. If false it will be month instead
   *
   * @param day defines the day of the month, when the transaction will be executed
   *
   * @author marcel mulch
   *
   * @date 25.11.2004
   */	
  HobitError createJobStartStandingOrder(bool skipBad,
                                                              int country,
                                                              const string &bankCode,
                                                              const string &accountId,
                                                              const string &accountSubId,
                                                              int otherCountry,
                                                              const string& otherBankCode,
                                                              const string& otherAccount,
                                                              const list<string> &otherName,
                                                              const list<string> &purpose,
                                                              const  Value&  value,
                                                              int textkey,
                                                              const Date& date,
                                                              const Date& firstdate,
                                                              const Date& lastdate,
                                                              const int turnus,
                                                              bool weekly,
                                                              int day);


   /** Deletes a standing order */
  HobitError createJobDeleteStandingOrder(Pointer<StandingOrder> sto);    
    
  /** Removes the data directory using a simple rm
  */ 
  void removeUncryptedEnvironment();

  /** Returns the name of the bank for the given id. Returns an empty string, if no bank could be found.
  */
  QString getBankNameForId(const QString & id);
  
  
  /** Returns the previously choosen customer */
  Pointer<Bank> getBank();
  
  /** Returns the bankId BLZ
  */
  QString HoBIT::getBankId();
  
  /** Returns the user from OpenHbci database.
  */
  Pointer<User> getUser();
  
  /** Returns the first customer from the user's customer list. Return value should be checked with isValid().
  */
  Pointer<Customer> getCustomer();
  
  /** Returns the currently used medium from OpenHbci database. Return value should be checked with isValid().
  */
  Pointer<Medium> getMedium();
  
  /** Returns true, if cardreader is up.
  */
  bool isCardReaderOk();
  
  /** Returns true, if daemon is running. Else tries to start daemon with sudo. If this fails, returns false.
  */
  bool isChipcardDaemonRunning();
  
  /** Import an RDH2 formatted key. Format is specified in 
    * "FinTS-Format der RDH2 Diskette" by PPI, Hamburg. see www.hbc-kernel.org
    */
  bool importRDH2Key(const QString & fileName, const QString & password);
  
  /** Returns true for acceptable and intermediate accountId strings
  */
  bool validateAccountIdString(const QString & accountId);
  
  /** Read property of AdminJobFactory m_AdminJobFactory. 
  */
  AdminJobFactory * HoBIT::getAdminJobFactory()const;
  
  /** Read property of AccountJobFactory m_AccountJobFactory. 
  */
  AccountJobFactory * HoBIT::getAccountJobFactory()const;
  
  /** returns the current HBCI mode */
  HBCI_MODE getHbciMode();
  
  /** Returns true, if no jobs are in job queues
  */
  bool areJobQueuesEmpty();
  
  /** write property for _CryptDir
  */
  void setCryptDir(const string & newVal);
  /** Read property of QString _CurrentAccount. 
  */
  const QString& HoBIT::getCurrentAccount()const;
  /** Read property of QString _CurrentAccountName. 
  */
  const QString& HoBIT::getCurrentAccountName()const;
  /** Write property of QString _CurrentAccount. 
  */
  void HoBIT::setCurrentAccount( const QString& _newVal);
  /** Returns temporary config data directory.
  */
  QString getConfigDataDir();

private:
  /**Creates the aqmoney engine
  */ 
   void createApi();
   /** Removes all the text from the beginning of startBlock to the end of endBlock from file fileName.
   */
   bool removeTextBlock(QString startBlock, QString endBlock, QString fileName);
  
  /** deletes the aqmoney engine
  */
  // void deleteApi();
   
private:
  //These are owned by HoBIT. Pointer<> types are neccessary to match the aqMoney API.
  //!!The Pointer<> class autodeletes the passed pointer!!
  Pointer<AqMoneyAPI> _api;
  Pointer<HoBIT_Interactor> _interactor;
  Pointer<HoBIT_Auth> _auth;
  Pointer<HoBIT_Validator> _validator;
  
  DatabaseAccess * _Database;
  
  //These are copies, which must not be deleted
  HoBIT_Connector * _Connector;
 // HoBIT_Crypt * _Crypt;
  
  
  /**aqmoney flag
  */
  bool _readonly;
  
  // qt dialogs
  QWidget* _parent;
  /** id of the current account
  */
  QString _CurrentAccount;
  
  /** Name (description) of the current account
  */
  QString _CurrentAccountName;
  
  // db directories
  /** OpenHci data
  */
  string  _openhbcifile;
  /** AqMoney data, including accounts
  */
  string  _aqmoneyfile;
  /** Transfer template data
  */
  string  _templatefile;
  /** home
  */
  string  _homedir;
  /** transaction data etc. GWEN database
  */
  string  _hbcidatadir;
  /**The top-directory for the temporary files
  */
  string _ConfigDataDir;
  /**The directory for encrypted files
  */
 // string _CryptDir;
  /** Hbci password
  */
  string  _password;
  
  /** Name of the program
  */
  string _applicationname;
  
  /** the list of all known banks.
  */
  list<HoBIT_Bank> _banklist;
  
  /** the list of all banks, that support hbci.
  */
  list<HoBIT_Bank> _hbcibanklist;
  
  
  // factories
  AccountJobFactory * _AccountJobFactory;
  AdminJobFactory * _AdminJobFactory;

};

#endif
